#!/usr/bin/env ruby
# Example code for using the MIME::MultipartAlternative::Lite::Body library.
# The idea is to demonstrate how you might use it to generate a series of
# E-mails from a set of recipients.  
# Two template filesa re used. One is in BlueCloth/Textile format, the other
# is XHTML.  A recipient can recievcew one or the other.
# The message content is slightly different, but they both use the same
# images.  Calling code ot encode an image file every time wuld be a waste, so
# the encoded version of each image is stored on disk.  To make this work, a
# subclass of MIME::MultipartAlternative::Lite::Body is used.  The subclass
# overrides the method +file_encode+ so that it looks in a class-level hash,
# and, if not found there, loads the encoded data from disk (adding it to the
# hash as well).
# For ecah recipient, the E-mail is constructed and mailed.  Change the E-mail
# settings to suit your own environment.  
#
require 'net/smtp'
require 'bluecloth'
require 'mime/multipart_alternative_lite'

# Helper method to send mail 
def send_mail( send_to_server , send_to , send_from_server, send_from,  msgstr    )
  send_to_set = send_to.split( ';').map { |e| e.strip }
  send_to_set.each do |send_to|
    Net::SMTP.start( "#{send_to_server}", 25, send_from_server ) do |smtp|
      smtp.send_message( msgstr, send_from, send_to )
    end
  end

end

# Subclass that lets us change how encoded image data is created
class HtmlDefault < MIME::MultipartAlternative::Lite::Body
  @@encoded_data  = {} 

  def initialize( encoded_file_dir, src_text )
    @encoded_file_dir  = encoded_file_dir 
    super( src_text  )
  end

  def file_encode( file_name )
    unless @@encoded_data[ file_name ]
      @@encoded_data[ file_name ] = create_encoding( file_name  )
    end
    return @@encoded_data[ file_name ]    
  end 

  def create_encoding( file_name  )
    File.read( "#@encoded_file_dir/#{file_name}.encoded")
  end
end


$current_dir = File.expand_path( File.dirname( __FILE__ ))

# A set of mail recipients. In real-life, this sort of data would come from
# a database.  You would probably have the templates there, too, but here
# they are stored as disk files.  Note that the code does not care what
# format is used for the template.  The Body class handles alternative
# content the details for us.
# Also note, though, that this example does not allow for differing
# plain-text and XHTML content, though that is an option provided by
# MIME::MultipartAlternative::Lite::Body
recipients = [
  { :SALUTATION  => "Mr.", 
    :FIRSTNAME   => "Haskell",  
    :LASTNAME    => "Curry", 
    :EMAIL       => "foo@example.com;  monad.britt@gmail.com", 
    :TOPIC       => "Code News" ,
    :DATE        => Time.now.to_s},

  { :SALUTATION  => "Mr.", 
    :FIRSTNAME   => "Foo",  
    :LASTNAME    => "bar", 
    :EMAIL       => "foobar.britt@gmail.com", 
    :TOPIC       => "Newsletter" ,
    :DATE        => Time.now.to_s}

]

# Bundle up the message with some mail headers.
def add_headers( msg, recpt, subject )
"Date: #{Time.new}
From: james@neurogami.com
Reply-To: james@neurogami.com
Organization: Neurogami LLC
MIME-Version: 1.0
To: #{recpt}
Subject: #{subject}
#{msg}"
end

# Make the mail and send it.  
recipients.each do |rcpt|
  msg_src = File.read( "#{$current_dir}/templates/#{rcpt[ :TOPIC ]}.tmpl")
  rcpt.each  {  |k, v|  msg_src .gsub!( /#{k}/, v )}

  # multipart = AlternateEncoding.new( "#{$current_dir}/encoded_images", msg_src , BlueCloth )
  multipart = HtmlDefault.new( "#{$current_dir}/encoded_images", msg_src  )

  body = multipart.to_s
  send_mail( "smtp.someplace.net", 
            rcpt[ :EMAIL ], 
             "smtp.someplace.net", 
             "james@neurogami.com",  
             add_headers( body ,
                         rcpt[ :EMAIL ],
                         rcpt[ :TOPIC ].gsub( /_/, ' ') )
           )   

           sleep( 3 )
end

