#!/usr/bin/env ruby

require 'md5'
require 'rexml/parsers/pullparser'
require 'rexml/document'
require 'mime/types'

include  REXML::Parsers

module MIME 
  module MultipartAlternative
    module Lite


      EOL = "\n"
      EOL2 = EOL + EOL


        # Default class for converting plain text into XHTML.
        # Not very crafty; you will get better results by providing your own
        # conversion class and passing it in to the contructor for Body.
        # The class must have a cnstrutor that takes a string, and define a to_html
        # method that converts that string to (partial) XHTML.  The resulting markup
        # will get stuffed into wrapping html/body tags.
        #
        # See how BlueCloth works for a goood example.
      class PlainTextConverter
        def initialize( str='' )
           @str = str
        end

          # Does what you woud think, though does not do anything clever.  
        def to_html( lite=nil )
          "<html>\n <body>\n  <pre>#{@str}</pre>\n </body>\n</html>"
        end
      end

        # Basically a Structlike thing with some helper methods.
        # Calling to_s returns the header section for the E-mail
      class MailHeader
         attr_accessor  :from, :date, :reply_to, :organization, :mime_version, :to, :subject

         def initialize( value_hash = nil )  # :yield: c
           @date =  Time.new.to_s 
           @mime_version = '1.0'
           value_hash.each { |k, v| set( k, v ) } if value_hash 
           yield self if block_given?
         end

         def set( key, value )
           begin
             self.send( "#{key}=", value )
           rescue Exception
             return nil
           end
           value
         end

         def to_s
           @reply_to.to_s.strip.size < 1 ?   reply_to = @from : reply_to = @reply_to
           s =  "Date: #{@date}" + EOL
           s << "From: #{@from}" + EOL
           s << "Reply-To: #{reply_to}" + EOL
           s << "Organization: #{@organization}" + EOL
           s << "MIME-Version: #{@mime_version}" + EOL
           s << "To: #{to}" + EOL
           s << "Subject: #{subject}"
         end

       end
        


        # Provides a way to take either a plain-text or XHTML message and create an MHTML
        # string suitable for E-mail.  The resulting E-mail source will contain both
        # a plain-text and XHTML version of the content; conversion from one form to
        # the other, depending on what was given to the class, is done
        # automatically.
        # 
        # If the message body contains +img+ elements, and the +src+ attribute has a
        # relative path, the class assumes this means the referenced image can be found
        # on the local file system, using that relative path.  This file is slurped
        # in, uuencoded, and attached as a related multipart segment.  The image +src+
        # value is then replaced with an MHTML +cid:+ reference.  This allows images
        # to be embedded in the mail.  You may of course use fully-qualified http
        # URLS for images; these values will remain unchanged.
        # 
        # The class constructor takes a message source string, a hash holdng header
        # name=value pairs, and a plain-text-to-XHTML class constant.  All are
        # optional, though the first two must be provided at some point before
        # generating the E-mail.  (See the docs for PlainTextConverter for more
        # details on what the converter class must do.)
        #
        # This class does not do any mailing; it knows nothing about SMTP or sendmail
        # or anything other than creating an MHTML string.  There are no options for
        # omitting either the plain-text or XHTML version of the message.  It relies
        # on the MIME::Types lib for determiing the MIME type of any embeded images;
        # despite this, it knows nothing about any other sort of embedded content.
        # You cannot embed Flash or AVI files or anyhting other than img content.
        #
        # Consider the code alpha.  Features may come and go.  The API may change.
        # Ideally, it will get better.
      class Body
        SRC_HTML        = 1
        SRC_PLAIN_TEXT  = 2
        SRC_PRESET      = 3
        BOUNDRY_PREFIX = '--'

        attr_accessor :plain_text, :html_text

        def determine_src_type_from_given_content
          return Body::SRC_PRESET if ( @html_text.to_s.size > 0 ) and ( @plain_text.to_s.size > 0  )
          return nil unless @src_text.to_s.strip.size > 0
          return Body::SRC_HTML   if @src_text =~ /<html>/i
          return Body::SRC_PLAIN_TEXT
        end

          # Pass in the body text and a converter class that turns plain-text into HTML.
          # THe plain-text converter argument must be a Class object that may be used
          # to instantiate an object that implements new( plain-text ) and to_html(
          # lite=nil ).  THe BlueCloth class is a perfect example.  The results of
          # to_html should be proper XHTML markup such that it can be correctly
          # inserted into a 'body' element. Again, see what BlueCloth does.
          # If your pass in plain-text as the source text, the code uses that as the
          # 'alternative' view of the document, then uses the plain-text converter
          # class to turn it into XHTML.  THere is a built-in default class if you
          # decide not to supply your own.
          # If you pass in XHTML as the source text, then the code attempts to convert
          # the markup into a suitable plain-text alternative version.
          # (Ideally, the passed-in converter class should be able to convert both
          # ways, but ... )
          # Note that the markup must be valid XML, i.e., XHTML or HTML with no
          # unclosed elements or goofy attributes.  Some basic testing suggests that
          # HTML mail readers should have no problem with the XHTML.
          # Of course, if trouble is reported then this may have to change.
        def initialize( src_text=nil, plain_text_converter = nil )
          @src_text = src_text
          @plain_text_converter = plain_text_converter || PlainTextConverter
          @plain_text = ''
          @html_text = ''
          @guid_count = 0
          @main_boundry_guid = create_boundry_guid
          @related_boundry_guid = create_boundry_guid( 5 )
           yield( self ) if block_given? 
          @src_type  = determine_src_type_from_given_content
          construct_alternate_content  
        end

        def html_to_plain
          @plain_text = xml_to_plaintext( @html_text )  if @html_text.to_s.strip.size > 0
        end
        
        def plain_to_html
          @html_text = '<html><body>' + @plain_text_converter.new( @src_text ).to_html( ) + '</body></html>'
        end
        

          # Needs work.  Should, perhaps, wrap obvious text with Textile or 
          # Markdown or something like that.  Or allow the client to pass in the
          # converter class, as is doen for converting plain-text to XHTML
        def xml_to_plaintext xhtml_text
          s  = ''
          parser = PullParser.new( xhtml_text )
          while parser.has_next?
            pull_event = parser.pull
            if pull_event.text?     
               s  <<  pull_event[0]  + ' '
            end
          end
          s
        end

        def set_header( k, v)
          @headers.set( k, v ) 
        end
        
        def set_headers( h_vals )
          h_vals.each { |k, v| set_header( k, v) }
        end

        def header_str
          @headers.to_s
        end

        def construct_alternate_content
          case @src_type  
            when Body::SRC_PRESET
              return Body::SRC_PRESET
            when Body::SRC_HTML
              @html_text = @src_text
              html_to_plain
              return Body::SRC_HTML 
            when Body::SRC_PLAIN_TEXT
              @plain_text  = @src_text
              plain_to_html
              return Body::SRC_PLAIN_TEXT 
            else
              @plain_text = @src_text
              @html_text  = @src_text
              return Body::SRC_PLAIN_TEXT              
          end    
        end

          # This was an early shot at a simple API but may change or go away.
          # One can pass in all the needed parts when initializing the class, and
          # then call to_s.  It can be handy, though, to have another way to alter
          # the E-mail resources and build the text on-demand.
        def make_mail   src_text = '', header_hash = nil, plain_text_converter = nil 
          @plain_text_converter = plain_text_converter if plain_text_converter
          set_headers header_hash if  header_hash
          @src_text = src_text
          construct_alternate_content    
          to_s
        end

          # Loops over the source XHTML, locates the img elements, and creates a
          # hash of relative src URLs and id values.  Tries to avoid duplication; if
          # two img elements have the same relative img src then they should have
          # the same cid reference, and the referenced image should only be embedded
          # once.
        def inline_hash 
          h = {}
          return h unless @html_text.to_s.strip.size > 2
          doc = REXML::Document.new( @html_text )
          cid_count = 0
          
          doc.elements.to_a( "//img" ).each_with_index do |img, idx|
            
            src = img.attributes[ 'src' ]
            unless src =~ /^http/
              h[ src ] = make_cid( src , cid_count )
              cid_count += 1
            end
          end
          
          doc.elements.to_a( "//img" ).each_with_index do |img, idx|

            src = img.attributes[ 'src' ]
            if h[ src ]
              doc.elements.to_a( "//img" )[idx].attributes[ 'src' ] = "cid:#{h[ src ]}"
            end      
          end
          
          @html_text = "" 
          doc.write( @html_text , 2)
          mozilla_hack 
          h
        end
        
          # It appears that Mozilla mail, at least that which comes with
          # Mozilla 1.6, has a stupid bug: It will not render inline images if
          # the 'src' attribute value is quoted with single quotes.  They have
          # to be double quotes.  REXML is used to manipulate the source
          # XHTML, and it emits attributes quoted qith single quotes;  this
          # should amke NO difference to any application, but ...
          # Anyway, this hack replaces those src attribute quotes.
        def mozilla_hack 
          @html_text.gsub!( /src=\'([^'']*)\'/, 'src="\1"' )
        end
         
          # Creates what should be a unique ID for inline image referencing.  
          # Simplistic, but should be fine unless the message body is being 
          # intermixed with some other multipart content. 
        def make_cid src, cid_count, offset = 0
          "part#{cid_count}" # .#{MD5.new(rand.to_s + offset.to_s).hexdigest[0..8]}@neurogami.com"
        end
        
          # Generates the MIME multipart body 
        def to_s
          construct_alternate_content    
          s = "Content-Type: multipart/alternative;" + EOL 
          s << " boundary=\"#{@main_boundry_guid}\"" +   EOL2  
          s << "This is a multi-part message in MIME format."  + EOL 
          s << Body::BOUNDRY_PREFIX + @main_boundry_guid +  EOL 
          s << "Content-Type: text/plain; charset=us-ascii"  + EOL 
          s << "Content-Transfer-Encoding: 7bit"  + EOL2 
          s << @plain_text  + EOL2 
          s << Body::BOUNDRY_PREFIX + @main_boundry_guid  + EOL 
          
          # Not sure if we can omit this if the message has references to
          # encoded content.  Doesn't appear to break anyhting by including it
          # though, and just leaving it means less logic to code.
          s << "Content-Type: multipart/related;"  + EOL 
          s << " boundary=\"#{@related_boundry_guid}\""  + EOL2 + EOL 
          s << Body::BOUNDRY_PREFIX + @related_boundry_guid  + EOL 
          
          s << "Content-Type: text/html; charset=us-ascii"  + EOL 
          s << "Content-Transfer-Encoding: 7bit"  + EOL2 
          
          binary_chunk = ""
          inline_hash.each do |src, cid|
            binary_chunk << make_binary_chunk( @related_boundry_guid, src, cid )
          end          
          s << @html_text.strip  + EOL2
          s << binary_chunk
          
          # Omit this next line if no encoded segments? See comment above
          s << Body::BOUNDRY_PREFIX + @related_boundry_guid  + "--" +  EOL + EOL2     
          s << Body::BOUNDRY_PREFIX + @main_boundry_guid  + "--" +  EOL2 
        end

          # Givena  file name, returns the associated MIME type
        def file_content_mime_type file_name
           MIME::Types.of( File.extname( file_name )).to_s
        end

          # There is probably a better name for this; it creates a relared multpart
          # segment for a given source file (i.e., a local image file).
        def make_binary_chunk( boundry, src, cid )
          s =  "#{Body::BOUNDRY_PREFIX}#{boundry}"  + EOL     
          s << "Content-Disposition: inline; filename=\"#{src.gsub( /\\/, '/' )}\"" + EOL     
          s << "Content-ID: #{cid}"  + EOL          
          s << "Content-Transfer-Encoding: base64"  + EOL           
          s << "Content-Type: #{file_content_mime_type src}; name=\"#{src.gsub( /\\/, '/' )}\"" + EOL2
          s << file_encode( src  )  + EOL2
        end

          # Returns a GUID string for demarcation body chunks
        def create_boundry_guid( offset = 0 )
          "#{MD5.new(Time::now.to_s + rand.to_s + offset.to_s).hexdigest[1..24]}"
           @guid_count += 1
           "boundry_#{@guid_count}"
        end
        
          # Given a file name, retun an encoded verions of the file contents. 
        def file_encode( file_name )
          raise "Cannot find file '#{file_name}' " unless File.exist?( file_name )
          begin
            content = File.open( file_name, "rb") {|f| f.read}
          rescue Exception
            raise "Error opeing file '#{file_name}': #{$!}"
          end
          content = [content].pack("m*").split("\n")
          content.collect { |line| line.dump}.join( "\n" ).gsub( /\"/, '' ) 
        end
      end

        # Wrapper class to create a complete E-mail, with simple headers and a
        # multipart/alternative message body.  Essentially delegates to the
        # Body class.
        # The caller may either pass in the message text and a hash holding
        # the header name=value pairs, as well as an optional plain-text
        # converter class, or create an instance by calling the contructor with
        # a block.  If the latter form is used, the constructor does not check
        # that a message body was included.
      class HtmlMail
         
        attr_accessor  :message, :plain_text_converter
         
        def initialize( src_text = nil, 
                        header_hash = nil, 
                        plain_text_converter = nil ) # :yield: c
                        
          if block_given?
            @header = MailHeader.new(  )            
            yield self 
            @body = Body.new( @message, @plain_text_converter  ) 
          else
            raise "You must pass in some message text if you do not initialize using a block!" unless src_text 
            raise "You must pass in a header hash if you do not initialize using a block!" unless header_hash
            @plain_text_converter  = plain_text_converter
            @message = src_text
            @body = Body.new( src_text, plain_text_converter  ) 
            @header = MailHeader.new( header_hash )
          end
         
          def method_missing( sym , *args )
             args.size > 0 ? @header.send( sym.id2name, args ) : @header.send( sym.id2name  ).to_s
          end

          def to_s
            "#@header\n#@body"
          end
        end
      end      
    end
  end
end

# ===Admin Stuff
# Author: James Britt <jamesgb@neurogami.com>
#
# License: Copyright (c) 2004 James Britt and Neurogami, LLC. Released under the same license as Ruby.
#
# ===Warranty
# This software is provided "as is" and without any express or implied warranties, including, without limitation, the implied warranties of merchantibility and fitness for a particular purpose.
