#!/bin/env ruby

$:.unshift File.expand_path( File.dirname( __FILE__ ) )

require "test/unit"
require 'rexml/document'
require 'mime/multipart_alternative_lite'
require 'bluecloth'



class TC_MimeMultiPart < Test::Unit::TestCase

  def setup
    # FILL IN THESE VALUES WITH SOMETHING USEFUL!
    $from_email  = 'devel@neurogami.com'
    $to_email    = 'foo@example.com'
    $reply_to    =  'devel@neurogami.com'
  end

  def tear_down

  end

  def xtest_block_constructor
    body  = MIME::MultipartAlternative::Lite::Body.new(  ) { |b| 
      b.plain_text  = "Your mail reader does not support HTML!!"
      b.html_text  = source_html

    }
    assert_equal( MIME::MultipartAlternative::Lite::Body::SRC_PRESET, body.construct_alternate_content )    
    assert_equal( MIME::MultipartAlternative::Lite::Body, body.class )    
    assert_equal( "BOGUS", body.to_s )        
  end



  def test_mail_creation
    vals = {}
    vals[ 'to' ] = $to_email
    vals[ 'from' ] =  $from_email 
    vals[ 'reply_to' ] = $reply_to 
    vals[ 'organization' ] = 'Neurogami'
    vals[ 'subject' ] = 'Test mail'
    
    mail = MIME::MultipartAlternative::Lite::HtmlMail.new( "nothing", vals ) 
    assert_equal( MIME::MultipartAlternative::Lite::HtmlMail, mail.class )    

    mail = MIME::MultipartAlternative::Lite::HtmlMail.new do |m|
       m.from         = $from_email
       m.date         = Time.new.to_s
       m.reply_to     = $reply_to
       m.organization = 'Neurogami'
       m.to           = $to_email
       m.subject      = 'This is a test'
       m.message      =  source_html # 'This is my message'
     end

    assert_equal( MIME::MultipartAlternative::Lite::HtmlMail, mail.class )
    assert_equal( $from_email, mail.from )    
    assert_equal( expected_mail_01, mail.to_s )         
  end

 # FOO  
  def xtest_make_mail
    expect = "NEED TO FIX THIS PART"
    vals = {}
    
    mail = MIME::MultipartAlternative::Lite::HtmlMail.new do |m|
       m.from         = $from_email
       m.date         = Time.new.to_s
       m.reply_to     = $reply_to
       m.organization = 'Neurogami'
       m.to           = $to_email
       m.subject      = 'This is a test'
       m.message      = source_html
     end
    

    res = mail.to_s 
    File.open( "out.eml", 
               File::TRUNC|File::RDWR|File::CREAT ) { |f|
               f.print res  }
    assert_equal( expect,  res  )        
  end

  def xtest_markdown
    body  = MIME::MultipartAlternative::Lite::Body.new( source_markdown_01, BlueCloth ) 
    res = body.to_s 


    # This is to create a source file that we can feed to some mailer app to
    # see how the results look in an actual mail program.
    # No, it's not a real unit test, but the eyeballing the results is
    # important
    File.open( "out2.eml", 
               File::TRUNC|File::RDWR|File::CREAT ) { |f|
               f.print res  }    
    
               
    expected_lines =  expected_markdown_01.split( "\n" )
    res_lines = res.split( "\n" )
    
    res_lines.each_index do |x|
      assert_equal( expected_lines[ x ] , res_lines[ x ]  )              
    end
    
  end

  def source_markdown_01
'This is the heading
-------------------

No picture in *this* test mail!

A list just to see that the conversion worked:
1. one
2. two
3. three'
    
  end

  def expected_markdown_01
    'Content-Type: multipart/alternative;
 boundary="boundry_1"

This is a multi-part message in MIME format.
--boundry_1
Content-Type: text/plain; charset=us-ascii
Content-Transfer-Encoding: 7bit

This is the heading
-------------------

No picture in *this* test mail!

A list just to see that the conversion worked:
1. one
2. two
3. three

--boundry_1
Content-Type: multipart/related;
 boundary="boundry_2"


--boundry_2
Content-Type: text/html; charset=us-ascii
Content-Transfer-Encoding: 7bit

<html><body><h2>This is the heading</h2>

<p>No picture in <em>this</em> test mail!</p>

<p>A list just to see that the conversion worked:</p>

<ol>
<li>one</li>
<li>two</li>
<li>three</li>
</ol></body></html>

--boundry_2--


--boundry_1--
'
  end
  
  def source_html
%~<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/html4/loose.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta http-equiv="content-type" content="text/html; charset=iso-8859-1">
    <title>Workspace</title>
    <meta name="generator" content="amaya 9.2.1, see http://www.w3.org/Amaya/">
    <style type="text/css">
      <!--
		#topnav { background:#1a4168; color:#ccc;}
#topnav a:link, #topnav a:visited, #topnav a:hover {
	color:#fff;
	text-decoration:none;
	border-bottom:0px;
}
#topnav a:hover { border-bottom:1px dotted #fff;}

/* THE CURR_NAV STYLE FOR THE TOPNAV */
#topnav a:link.curr_nav, #topnav a:visited.curr_nav, #topnav a:hover.curr_nav {
	color:#ccc !important;
	border-bottom:0px;
	}
.asterisk {
	color:#ccc;
	font:bold 14px/10px verdana,arial,helvetica sans-serif;
	margin:0 9px;
}
      .topText {font-size:10px; color:#003399; line-height:25px;}
      .title {font-size:16px; font-weight:bold; color:#000000;}
      .subTitle {font-size:13px; font-weight:bold; color:#000000;}
      p {font-size:12px; color:#666666; line-height:20px; font-family: verdana;}
      .footerText {font-size:11px; color:#666666;}
      .headerBanner {font-size: 18px; font-weight: bold;}
      td {font-family: verdana;}
      -->
    </style>
	<!--
	 We're having a problem with flow.  We don't know how far down the bolg content will go,
	 so it's hard to use tables for this.  We end up with
	 http://www.originalsignal.com/ uses rows of three; they are expandy, but never wrap.
	 
	 
	-->
  </head>
  <body bgcolor="#6B94B0">
    <table width="100%" bgcolor="#6B94B0" cellpadding="0" cellspacing="0">
      <tbody>
        <tr>
          <td valign="top" align="center">
            <table width="80%" bgcolor="#FFFFFF" cellpadding="10px" cellspacing="0">
              <tbody>
              <tr>
               <td valign="top" align="center" colspan='2' width="100%" id='topnav'><a class="curr_nav" href="http://davenetics.com">home</a>
	<span class="asterisk">*</span>
	<a  href="http://davenetics.com/category/politics/">politics</a>
	<span class="asterisk">*</span>
	<a  href="http://davenetics.com/category/media/">media</a>
	<span class="asterisk">*</span>
	<a  href="http://davenetics.com/category/musings/">musings</a></td>     
                </tr>
                <tr align="left" valign="top" >
                  <td width='50%' colspan="2"><span style='width: 400px; background-color: #FFFFCC; float:left;' ><h2>My New Entry</h2>


	<p>Lorem ipsum mea suas senserit et! Eos ex modus fugit placerat, ex vix ludus offendit. Quo ut suas vulputate, at est ipsum offendit. His in diam argumentum?</p>


	<p>Eros audiam est ad! Ne eos laudem fabellas abhorreant, eos minim verear signiferumque ex, graeco verear democritum te mel. Cu decore nonummy honestatis mel, ex error civibus pro. Usu debitis patrioque in, at ius feugiat indoctum deseruisse. Vis te velit tation libris, id movet luptatum mea!</p>


	<p>No cum nonummy scriptorem, eum dicam facete inimicus te. No commodo fastidii mea, sale affert saperet vel at. Simul vocibus appetere ei mea, eu persius ocurreret efficiantur mei? At modus omittam vim, ornatus ocurreret repudiare mel ex, sit nonumy ridens et.</p>


	<p>Lorem ipsum mea suas senserit et! Eos ex modus fugit placerat, ex vix ludus offendit. Quo ut suas vulputate, at est ipsum offendit. His in diam argumentum?</p>


	<p>Eros audiam est ad! Ne eos laudem fabellas abhorreant, eos minim verear signiferumque ex, graeco verear democritum te mel. Cu decore nonummy honestatis mel, ex error civibus pro. Usu debitis patrioque in, at ius feugiat indoctum deseruisse. Vis te velit tation libris, id movet luptatum mea!</p>


	<p>No cum nonummy scriptorem, eum dicam facete inimicus te. No commodo fastidii mea, sale affert saperet vel at. Simul vocibus appetere ei mea, eu persius ocurreret efficiantur mei? At modus omittam vim, ornatus ocurreret repudiare mel ex, sit nonumy ridens et.</p></span>
				  <span style='width: 320px; height: 400px; float: left; margin:10px;' >
   <span style='font-size: 16px'></span><br > </span>
<span style='width: 320px; height: 400px; float: left; margin:10px;' >
   <span style='font-size: 16px'>Solution Watch</span><br ><a href='' title='Back to School with the Class of Web 2.0: Part 1' >Back to School with the Class of Web 2.0: Part 1</a><br>
<a href='' title='Squirl - Organize and Share Your Collections' >Squirl - Organize and Share Your Collections</a><br>
<a href='' title='SimpleSeating - Web 2.0… Seating Charts?' >SimpleSeating - Web 2.0… Seating Charts?</a><br>
<a href='' title='Tracking the web with Single Page Aggregators' >Tracking the web with Single Page Aggregators</a><br>
<a href='' title='Menutopia - Restaurant Review Community' >Menutopia - Restaurant Review Community</a><br>
<a href='' title='Crazy Egg Launched - Visualize Visitor Clicks' >Crazy Egg Launched - Visualize Visitor Clicks</a><br>
<a href='' title='Simple Social Networking with Loowa' >Simple Social Networking with Loowa</a><br>
<a href='' title='Traineo for Weight Loss & Fitness' >Traineo for Weight Loss & Fitness</a><br>
<a href='' title='Del.icio.us Redesigns, Again. Plus Contextual Ads' >Del.icio.us Redesigns, Again. Plus Contextual Ads</a><br> </span>
<span style='width: 320px; height: 400px; float: left; margin:10px;' >
   <span style='font-size: 16px'>TechCrunch</span><br ><a href='' title='All Women Team Takes Yahoo Hack Day Top Prize' >All Women Team Takes Yahoo Hack Day Top Prize</a><br>
<a href='' title='New Look For Netvibes' >New Look For Netvibes</a><br>
<a href='' title='LiveJournal pushes ad sponsored communities, features' >LiveJournal pushes ad sponsored communities, features</a><br>
<a href='' title='Shutterfly takes photo printing to the bank in IPO' >Shutterfly takes photo printing to the bank in IPO</a><br>
<a href='' title='Yahoo’s BBAuth Will Allow Better Mashups' >Yahoo’s BBAuth Will Allow Better Mashups</a><br>
<a href='' title='DAG Ventures leads $15m more for Podshow' >DAG Ventures leads $15m more for Podshow</a><br>
<a href='' title='Why The New .Mac Webmail Is Important' >Why The New .Mac Webmail Is Important</a><br>
<a href='' title='Google Reader steps it up with new version' >Google Reader steps it up with new version</a><br>
<a href='' title='Facebook to put viral ads in your news feeds?' >Facebook to put viral ads in your news feeds?</a><br> </span>
<span style='width: 320px; height: 400px; float: left; margin:10px;' >
   <span style='font-size: 16px'>Mashable</span><br ><a href='' title='VideoRonk - Download Videos From YouTube, Google Video and Metacafe' >VideoRonk - Download Videos From YouTube, Google Video and Metacafe</a><br>
<a href='' title='Xanga Audio Launches' >Xanga Audio Launches</a><br>
<a href='' title='iBloks Launches Movie Maker for MySpace' >iBloks Launches Movie Maker for MySpace</a><br>
<a href='' title='Wanted: Moron to Buy YouTube' >Wanted: Moron to Buy YouTube</a><br>
<a href='' title='MyWorld: eBay Becomes MySpace' >MyWorld: eBay Becomes MySpace</a><br>
<a href='' title='MusicHawk’s MySpace for Music' >MusicHawk’s MySpace for Music</a><br>
<a href='' title='Orkut Woes, Google Video Gets Educational' >Orkut Woes, Google Video Gets Educational</a><br>
<a href='' title='AccuWeather’s Weather Badges for MySpace' >AccuWeather’s Weather Badges for MySpace</a><br>
<a href='' title='Facebook to Launch Advertising Through News Feed' >Facebook to Launch Advertising Through News Feed</a><br> </span>
<span style='width: 320px; height: 400px; float: left; margin:10px;' >
   <span style='font-size: 16px'>ProgrammableWeb</span><br ><a href='' title='More APIs: Fraud, Dots, and Stocks' >More APIs: Fraud, Dots, and Stocks</a><br>
<a href='' title='Latest APIs: Spam, Invoices and Ads' >Latest APIs: Spam, Invoices and Ads</a><br>
<a href='' title='30 Games Mashups' >30 Games Mashups</a><br>
<a href='' title='Books for Amazon Mashups' >Books for Amazon Mashups</a><br>
<a href='' title='NYC in the Movies Mashup' >NYC in the Movies Mashup</a><br>
<a href='' title='1001 Mashups' >1001 Mashups</a><br>
<a href='' title='Flickr Mashup Books' >Flickr Mashup Books</a><br>
<a href='' title='Mashup Contests and the diGGGadget' >Mashup Contests and the diGGGadget</a><br>
<a href='' title='Latest Mashup Contest News' >Latest Mashup Contest News</a><br> </span>
<span style='width: 320px; height: 400px; float: left; margin:10px;' >
   <span style='font-size: 16px'>Dion Hinchcliffe</span><br ><a href='' title='Seven Things Every Software Project Needs to Know About Ajax' >Seven Things Every Software Project Needs to Know About Ajax</a><br>
<a href='' title='Web 2.0 for Business: Innovation, The New New Internet, and Change' >Web 2.0 for Business: Innovation, The New New Internet, and Change</a><br>
<a href='' title='Media 2.0 and The World of Online Video - Never Mind the Quality see the Marketshare' >Media 2.0 and The World of Online Video - Never Mind the Quality see the Marketshare</a><br>
<a href='' title='All We Got Was Web 1.0, When Tim Berners-Lee Actually Gave Us Web 2.0' >All We Got Was Web 1.0, When Tim Berners-Lee Actually Gave Us Web 2.0</a><br>
<a href='' title='FOO Camp 06: Plenty of Smart People, Self-Organization, and Web 2.0 Goodness' >FOO Camp 06: Plenty of Smart People, Self-Organization, and Web 2.0 Goodness</a><br>
<a href='' title='Thinking Beyond Web 2.0: Social Computing and the Internet Singularity' >Thinking Beyond Web 2.0: Social Computing and the Internet Singularity</a><br>
<a href='' title='Windows Live and Kicking' >Windows Live and Kicking</a><br>
<a href='' title='Handicapping The Next Big Web 2.0 Sites for 2006' >Handicapping The Next Big Web 2.0 Sites for 2006</a><br>
<a href='' title='Creating Web 2.0 Applications: Seven Ways to Fully Embrace the Network' >Creating Web 2.0 Applications: Seven Ways to Fully Embrace the Network</a><br> </span>
<span style='width: 320px; height: 400px; float: left; margin:10px;' >
   <span style='font-size: 16px'>37signals</span><br ><a href='' title='Sudo sandwich' >Sudo sandwich</a><br>
<a href='' title='Fly on the wall: &quot;What does this button do?&quot;' >Fly on the wall: &quot;What does this button do?&quot;</a><br>
<a href='' title='The start of the Backpack calendar API' >The start of the Backpack calendar API</a><br>
<a href='' title='Crazyegg = crazy good' >Crazyegg = crazy good</a><br>
<a href='' title='Jitterbug and the Cellphone Simplicity Derby' >Jitterbug and the Cellphone Simplicity Derby</a><br>
<a href='' title='Audi R8' >Audi R8</a><br>
<a href='' title='Cha-Ching gets real' >Cha-Ching gets real</a><br>
<a href='' title='Fall foliage' >Fall foliage</a><br>
<a href='' title='&quot;Make it work&quot;' >&quot;Make it work&quot;</a><br> </span>
				  </td>
                </tr>
              </tbody>
            </table>
          </td>
        </tr>
      </tbody>
    </table>
  </body>
</html>
~
  end

  def expected_mail_01
    'Date: Thu Jul 29 14:45:55 US Mountain Standard Time 2004
From: ' + $reply_to + '
Reply-To: ' + $reply_to + '
Organization: Neurogami
MIME-Version: 1.0
To: ' + $to_email + '
Subject: This is a test

Content-Type: multipart/alternative;
 boundary=\"boundry_1\"

This is a multi-part message in MIME format.
--boundry_1
Conten t-Type: text/plain; charset=us-ascii
Content-Transfer-Encoding: 7bit

THis is my message

--boundry_1
Content-Type: multipart/related;
 boundary=\"boundry_2\"


--boundry_2
Content-Type: text/html; charset=us-ascii
Content-Transfer-Encoding: 7bit

<html>
    <body>
      <html>
 <body>
  <pre>THis is my message</pre>
 </body>
</html>
  </body>
  </html>

--boundry_2--


--boundry_1--


'
  end

end

