#!/bin/env ruby

$:.unshift File.expand_path( File.dirname( __FILE__ ) )

require "test/unit"
require 'rexml/document'
require 'mime/multipart_alternative_lite'
require 'bluecloth'



class TC_MimeMultiPart < Test::Unit::TestCase

  def setup
    # FILL IN THESE VALUES WITH SOMETHING USEFUL!
    $from_email  = 'devel@neurogami.com'
    $to_email    = 'foo@example.com'
    $reply_to    =  'devel@neurogami.com'
  end

  def tear_down

  end

  def test_block_constructor
    body  = MIME::MultipartAlternative::Lite::Body.new(  ) { |b| 
      b.plain_text  = "Your mail reader does not support HTML!!"
      b.html_text  = source_html

    }
    assert_equal( MIME::MultipartAlternative::Lite::Body::SRC_PRESET, body.construct_alternate_content )    
    assert_equal( MIME::MultipartAlternative::Lite::Body, body.class )    
    assert_equal( "BOGUS", body.to_s )        
  end



  def test_mail_creation
    vals = {}
    vals[ 'to' ] = $to_email
    vals[ 'from' ] =  $from_email 
    vals[ 'reply_to' ] = $reply_to 
    vals[ 'organization' ] = 'Neurogami'
    vals[ 'subject' ] = 'Test mail'
    
    mail = MIME::MultipartAlternative::Lite::HtmlMail.new( "nothing", vals ) 
    assert_equal( MIME::MultipartAlternative::Lite::HtmlMail, mail.class )    

    mail = MIME::MultipartAlternative::Lite::HtmlMail.new do |m|
       m.from         = $from_email
       m.date         = Time.new.to_s
       m.reply_to     = $reply_to
       m.organization = 'Neurogami'
       m.to           = $to_email
       m.subject      = 'This is a test'
       m.message      =  'THis is my message'
     end

    assert_equal( MIME::MultipartAlternative::Lite::HtmlMail, mail.class )
    assert_equal( $from_email, mail.from )    
    assert_equal( expected_mail_01, mail.to_s )         
  end

  
  def test_make_mail
    expect = "NEED TO FIX THIS PART"
    vals = {}
    
    mail = MIME::MultipartAlternative::Lite::HtmlMail.new do |m|
       m.from         = $from_email
       m.date         = Time.new.to_s
       m.reply_to     = $reply_to
       m.organization = 'Neurogami'
       m.to           = $to_email
       m.subject      = 'This is a test'
       m.message      = source_html
     end
    

    res = mail.to_s 
    File.open( "out.eml", 
               File::TRUNC|File::RDWR|File::CREAT ) { |f|
               f.print res  }
    assert_equal( expect,  res  )        
  end

  def test_markdown
    body  = MIME::MultipartAlternative::Lite::Body.new( source_markdown_01, BlueCloth ) 
    res = body.to_s 


    # This is to create a source file that we can feed to some mailer app to
    # see how the results look in an actual mail program.
    # No, it's not a real unit test, but the eyeballing the results is
    # important
    File.open( "out2.eml", 
               File::TRUNC|File::RDWR|File::CREAT ) { |f|
               f.print res  }    
    
               
    expected_lines =  expected_markdown_01.split( "\n" )
    res_lines = res.split( "\n" )
    
    res_lines.each_index do |x|
      assert_equal( expected_lines[ x ] , res_lines[ x ]  )              
    end
    
  end

  def source_markdown_01
'This is the heading
-------------------

No picture in *this* test mail!

A list just to see that the conversion worked:
1. one
2. two
3. three'
    
  end

  def expected_markdown_01
    'Content-Type: multipart/alternative;
 boundary="boundry_1"

This is a multi-part message in MIME format.
--boundry_1
Content-Type: text/plain; charset=us-ascii
Content-Transfer-Encoding: 7bit

This is the heading
-------------------

No picture in *this* test mail!

A list just to see that the conversion worked:
1. one
2. two
3. three

--boundry_1
Content-Type: multipart/related;
 boundary="boundry_2"


--boundry_2
Content-Type: text/html; charset=us-ascii
Content-Transfer-Encoding: 7bit

<html><body><h2>This is the heading</h2>

<p>No picture in <em>this</em> test mail!</p>

<p>A list just to see that the conversion worked:</p>

<ol>
<li>one</li>
<li>two</li>
<li>three</li>
</ol></body></html>

--boundry_2--


--boundry_1--
'
  end
  
  def source_html
'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
  <meta content="text/html;charset=ISO-8859-1" http-equiv="Content-Type" />
  <title>Test HTML</title>
</head>
<body bgcolor="#ffffff" text="#000000">
<p>This email should have two <font color="#339999">pictures</font>.&#160;
Here is the first one:</p>

<p>
<img height="8" width="8" alt="X1" src="images/test.jpg"/>
</p>
<p>And the other</p>

  <p>The <i>end</i></p>
 </body>
</html>'
  end

  def expected_mail_01
    'Date: Thu Jul 29 14:45:55 US Mountain Standard Time 2004
From: ' + $reply_to + '
Reply-To: ' + $reply_to + '
Organization: Neurogami
MIME-Version: 1.0
To: ' + $to_email + '
Subject: This is a test

Content-Type: multipart/alternative;
 boundary=\"boundry_1\"

This is a multi-part message in MIME format.
--boundry_1
Conten t-Type: text/plain; charset=us-ascii
Content-Transfer-Encoding: 7bit

THis is my message

--boundry_1
Content-Type: multipart/related;
 boundary=\"boundry_2\"


--boundry_2
Content-Type: text/html; charset=us-ascii
Content-Transfer-Encoding: 7bit

<html>
    <body>
      <html>
 <body>
  <pre>THis is my message</pre>
 </body>
</html>
  </body>
  </html>

--boundry_2--


--boundry_1--


'
  end

end

